@@ -1,10 +1,15 @@
-import aalib
-import Image
+import os
 import urllib2
-import aalib, urllib2, Image
 from cStringIO import StringIO
 
-screen = aalib.AnsiScreen(width=60, height=30)
+import Image
+import aalib
+
+if os.getenv('TERM') == 'linux':
+    screen = aalib.LinuxScreen
+else:
+    screen = aalib.AnsiScreen
+screen = screen(width=60, height=30)
 fp = StringIO(urllib2.urlopen('http://python.org/favicon.ico').read())
 image = Image.open(fp).convert('L').resize(screen.virtual_size)
 screen.put_image((0, 0), image)
